from pathlib import Path

from darker.__main__ import modify_file
from darker.chooser import choose_lines
from darker.diff import diff_and_get_opcodes, opcodes_to_chunks
from darker.formatters.black_formatter import BlackFormatter
from darker.formatters.formatter_config import BlackCompatibleConfig
from darker.verification import ASTVerifier, BinarySearch
from darkgraylib.utils import TextDocument


def format_file(
    src: Path,
    from_line: int,
    to_line: int,
    config: str = None,
):
    black_config = BlackCompatibleConfig(config=config)
    edited = worktree_content = TextDocument.from_file(src)
    max_context_lines = len(edited.lines)
    minimum_context_lines = BinarySearch(0, max_context_lines + 1)
    edited_line_nums = list(range(from_line, to_line + 1))

    while not minimum_context_lines.found:
        formatter = BlackFormatter()
        formatter.config = black_config
        formatted = formatter.run(edited, src)
        opcodes = diff_and_get_opcodes(edited, formatted)
        black_chunks = list(opcodes_to_chunks(opcodes, edited, formatted))
        chosen = TextDocument.from_lines(
            choose_lines(black_chunks, edited_line_nums),
            encoding=worktree_content.encoding,
            newline=worktree_content.newline,
        )

        if ASTVerifier(edited).is_equivalent_to_baseline(chosen):
            minimum_context_lines.respond(True)
            modify_file(src, chosen)
        else:
            minimum_context_lines.respond(False)



from argparse import ArgumentParser
from pathlib import Path

from .format import format_file


def parse_args():
    parser = ArgumentParser()
    parser.add_argument("path", metavar="PATH", type=Path)
    parser.add_argument("--from", type=int, default=1, dest="from_line")
    parser.add_argument("--to", type=int, default=-1, dest="to_line")
    parser.add_argument("--config")
    args = parser.parse_args()
    return args


def main():
    args = parse_args()
    format_file(args.path, args.from_line, args.to_line, args.config)


if __name__ == "__main__":
    main()



from pathlib import Path

from darkest.format import format_file

ORIGINAL = """\
a = [1,2,3,]

b = [5,6,7,]

c = [8,9,0,]
"""

EXPECTED = """\
a = [1,2,3,]

b = [
    5,
    6,
    7,
]

c = [8,9,0,]
"""


def test_format_file(tmpdir):
    sample = tmpdir / "sample.py"
    sample.write(ORIGINAL)
    format_file(src=Path(sample), from_line=3, to_line=3)
    result = sample.read()
    assert result == EXPECTED
